@@ -12,6 +12,7 @@ main = fail $   "test infrastructure not built,"
 import qualified Darcs.Test.Misc
 import qualified Darcs.Test.Patch
 import qualified Darcs.Test.Email
+import qualified Storage.Hashed.Test
 
 import Control.Monad ( filterM )
 import Control.Exception ( SomeException )
@@ -37,6 +38,10 @@ import qualified Shelly
 doUnit :: IO [Test]
 doUnit = return unitTests
 
+-- | TODO make runnable in parallel
+doHashed :: IO [Test]
+doHashed = return Storage.Hashed.Test.tests
+
 -- | This is the big list of tests that will be run using testrunner.
 unitTests :: [Test]
 unitTests =
@@ -79,6 +84,10 @@ runtest' (ShellTest fmt _ _ dp da) srcdir =
      setenv "TESTBIN" (toTextIgnore (srcdir </> "tests" </> "bin"))
      setenv "DARCS_TESTING_PREFS_DIR" $ toTextIgnore $ wd </> ".darcs"
      setenv "EMAIL" "tester"
+     setenv "GIT_AUTHOR_NAME" "tester"
+     setenv "GIT_AUTHOR_EMAIL" "tester"
+     setenv "GIT_COMMITTER_NAME" "tester"
+     setenv "GIT_COMMITTER_EMAIL" "tester"
      setenv "DARCS_DONT_COLOR" "1"
      setenv "DARCS_DONT_ESCAPE_ANYTHING" "1"
      p <- get_env_text "PATH"
@@ -134,6 +143,7 @@ runtest :: ShellTest -> Sh Result
 runtest t =
  withTmp $ \dir -> do
   cp "tests/lib" dir
+  cp "tests/network/sshlib" dir
   cp ("tests" </> testfile t) (dir </> "test")
   srcdir <- pwd
   silently $ sub $ cd dir >> runtest' t (toTextIgnore srcdir)
@@ -204,7 +214,8 @@ findNetwork dp tdir diffAlgorithmsDefault repoFormatsDefault =
 -- harness
 -- ----------------------------------------------------------------------
 
-data Config = Config { failing :: Bool
+data Config = Config { hashed :: Bool
+                     , failing :: Bool
                      , shell :: Bool
                      , network :: Bool
                      , unit :: Bool
@@ -226,9 +237,10 @@ data Config = Config { failing :: Bool
 defaultConfig :: Annotate Ann
 defaultConfig
  = record Config{}
-     [ failing       := False    += help "Run the failing (shell) tests [no]"
+     [ hashed        := False    += help "Run hashed-storage tests [no]"
+     , failing       := False    += help "Run the failing (shell) tests [no]"
      , shell         := True     += help "Run the passing, non-network shell tests [yes]"
-     , network       := False    += help "Run the network shell tests [no]"
+     , network       := True     += help "Run the network shell tests [yes]"
      , unit          := True     += help "Run the unit tests [yes]"
      , myers         := False    += help "Use myers diff [no]"
      , patience      := True     += help "Use patience diff [yes]" += name "p"
@@ -298,7 +310,8 @@ run conf = do
     stests <- shelly $ if shell conf then findShell darcsBin (testDir conf) False diffAlgorithm repoFormat else return []
     utests <- if unit conf then doUnit else return []
     ntests <- shelly $ if network conf then findNetwork darcsBin (testDir conf) diffAlgorithm repoFormat else return []
-    defaultMainWithArgs (ftests ++ stests ++ utests ++ ntests) args
+    hstests <- if hashed conf then doHashed else return []
+    defaultMainWithArgs (ftests ++ stests ++ utests ++ ntests ++ hstests) args
        where
           exeSuffix :: String
 #ifdef WIN32
