@@ -21,6 +21,9 @@ module Darcs.UI.Commands.Help (
  printVersion,
  listAvailableCommands ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.UI.Flags
     ( DarcsFlag
     , environmentHelpEmail
@@ -41,12 +44,11 @@ import Darcs.UI.Commands
     , usage
     )
 import Darcs.UI.External ( viewDoc )
-import Darcs.Repository.Lock ( environmentHelpTmpdir, environmentHelpKeepTmpdir
-                             , environmentHelpLocks )
+import Darcs.Util.Lock ( environmentHelpTmpdir, environmentHelpKeepTmpdir
+                       , environmentHelpLocks )
 import Darcs.Patch.Match ( helpOnMatchers )
 import Darcs.Repository.Prefs ( boringFileHelp, binariesFileHelp, environmentHelpHome )
-import Darcs.Repository.Ssh ( environmentHelpSsh, environmentHelpScp, environmentHelpSshPort )
-import Darcs.Repository.External ( environmentHelpProtocols )
+import Darcs.Util.Ssh ( environmentHelpSsh, environmentHelpScp, environmentHelpSshPort )
 import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.Path ( AbsolutePath )
 import Control.Arrow ( (***) )
@@ -202,7 +204,6 @@ environmentHelp = [
  environmentHelpSshPort,
  environmentHelpProxy,
  environmentHelpProxyPassword,
- environmentHelpProtocols,
  environmentHelpTimeout]
 
 -- | This module is responsible for emitting a darcs "man-page", a
@@ -405,8 +406,8 @@ environmentHelpEditor = (["DARCS_EDITOR", "DARCSEDITOR", "VISUAL", "EDITOR"],[
 
 environmentHelpPager :: ([String], [String])
 environmentHelpPager = (["DARCS_PAGER", "PAGER"],[
- "Darcs will sometimes invoke a pager if it deems output to be too long",
- "to fit onscreen.  Darcs will use the pager specified by $DARCS_PAGER",
+ "Darcs will invoke a pager if the output of some command is longer",
+ "than 20 lines. Darcs will use the pager specified by $DARCS_PAGER",
  "or $PAGER.  If neither are set, `less` will be used."])
 
 environmentHelpTimeout :: ([String], [String])
