@@ -19,7 +19,10 @@
 
 module Darcs.UI.Commands.Send ( send ) where
 
-import Prelude hiding ( (^), catch )
+import Prelude ()
+import Darcs.Prelude
+
+import Prelude hiding ( (^) )
 
 import System.Exit
     ( exitSuccess
@@ -32,7 +35,7 @@ import System.IO.Error ( ioeGetErrorString )
 import System.IO ( hClose )
 import Control.Exception ( catch, IOException )
 import Control.Monad ( when, unless, forM_ )
-import Storage.Hashed.Tree ( Tree )
+import Darcs.Util.Tree ( Tree )
 import Data.List ( intercalate, isPrefixOf )
 #ifdef HAVE_HTTP
 import Data.List ( stripPrefix )
@@ -86,7 +89,7 @@ import Darcs.Repository ( PatchSet, Repository,
                           readRepo, readRecorded, prefsUrl, checkUnrelatedRepos )
 import Darcs.Patch.Set ( Origin )
 import Darcs.Patch.Apply( ApplyState )
-import Darcs.Patch ( RepoPatch, description, applyToTree, invert )
+import Darcs.Patch ( IsRepoType, RepoPatch, description, applyToTree, invert )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 import Darcs.Patch.Witnesses.Ordered
@@ -94,7 +97,7 @@ import Darcs.Patch.Witnesses.Ordered
     mapFL, mapFL_FL, lengthFL, nullFL )
 import Darcs.Patch.Bundle ( minContext, makeBundleN, scanContextFile, patchFilename )
 import Darcs.Repository.Prefs ( addRepoSource, getPreflist )
-import Darcs.Repository.External ( fetchFilePS, Cachable(..) )
+import Darcs.Util.External ( fetchFilePS, Cachable(..) )
 import Darcs.UI.External
     ( signString
     , sendEmailDoc
@@ -109,15 +112,14 @@ import Darcs.UI.External
     )
 import Darcs.Util.ByteString ( mmapFilePS, isAscii )
 import qualified Data.ByteString.Char8 as BC (unpack)
-import Darcs.Repository.Lock
+import Darcs.Util.Lock
     ( withOpenTemp
     , writeDocBinFile
     , readDocBinFile
     , removeFileMayNotExist
     )
 import Darcs.UI.SelectChanges
-    ( selectChanges
-    , WhichChanges(..)
+    ( WhichChanges(..)
     , selectionContext
     , runSelection
     )
@@ -233,7 +235,6 @@ patchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = parseFlags O.matchSeveral flags
-    , S.diffAlgorithm = O.PatienceDiff
     , S.interactive = isInteractive True flags
     , S.selectDeps = selectDeps flags
     , S.summary = hasSummary O.NoSummary flags
@@ -263,11 +264,11 @@ sendCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 sendCmd fps input_opts [""] = sendCmd fps input_opts []
 sendCmd (_,o) input_opts [unfixedrepodir] =
  withRepository (useCache input_opts) $ RepoJob $
-  \(repository :: Repository p wR wU wR) -> do
+  \(repository :: Repository rt p wR wU wR) -> do
   context_ps <- the_context input_opts
   case context_ps of
     Just them -> do
-        wtds <- decideOnBehavior input_opts (Nothing :: Maybe (Repository p wR wU wR))
+        wtds <- decideOnBehavior input_opts (Nothing :: Maybe (Repository rt p wR wU wR))
         sendToThem repository input_opts wtds "CONTEXT" them
     Nothing -> do
         repodir <- fixUrl o unfixedrepodir
@@ -289,9 +290,9 @@ sendCmd (_,o) input_opts [unfixedrepodir] =
           the_context (_:fs) = the_context fs
 sendCmd _ _ _ = impossible
 
-sendToThem :: (RepoPatch p, ApplyState p ~ Tree)
-           => Repository p wR wU wT -> [DarcsFlag] -> [WhatToDo] -> String
-           -> PatchSet p Origin wX -> IO ()
+sendToThem :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+           => Repository rt p wR wU wT -> [DarcsFlag] -> [WhatToDo] -> String
+           -> PatchSet rt p Origin wX -> IO ()
 sendToThem repo opts wtds their_name them = do
 #ifndef HAVE_MAPI
   -- Check if the user has sendmail or provided a --sendmail-cmd
@@ -313,7 +314,7 @@ sendToThem repo opts wtds their_name them = do
   pristine <- readRecorded repo
   let direction = if doReverse opts then FirstReversed else First
       context = selectionContext direction "send" (patchSelOpts opts) Nothing Nothing
-  (to_be_sent :> _) <- runSelection (selectChanges us') context
+  (to_be_sent :> _) <- runSelection us' context
   printDryRunMessageAndExit "send"
       (verbosity opts)
       (hasSummary O.NoSummary opts)
@@ -347,10 +348,10 @@ sendToThem repo opts wtds their_name them = do
     Nothing     -> sendBundle opts to_be_sent bundle fname wtds their_name
 
 
-prepareBundle :: forall p wX wY wZ. (RepoPatch p, ApplyState p ~ Tree)
-              => [DarcsFlag] -> PatchSet p Origin wZ
-              -> Either (FL (PatchInfoAnd p) wX wY)
-                        (Tree IO, (FL (PatchInfoAnd p) :\/: FL (PatchInfoAnd p)) wX wY)
+prepareBundle :: forall rt p wX wY wZ. (RepoPatch p, ApplyState p ~ Tree)
+              => [DarcsFlag] -> PatchSet rt p Origin wZ
+              -> Either (FL (PatchInfoAnd rt p) wX wY)
+                        (Tree IO, (FL (PatchInfoAnd rt p) :\/: FL (PatchInfoAnd rt p)) wX wY)
               -> IO Doc
 prepareBundle opts common e = do
   unsig_bundle <-
@@ -365,12 +366,12 @@ prepareBundle opts common e = do
                                       (mapFL_FL hopefully to_be_sent)
   signString (parseFlags O.sign opts) unsig_bundle
 
-sendBundle :: forall p wX wY . (RepoPatch p, ApplyState p ~ Tree)
-           => [DarcsFlag] -> FL (PatchInfoAnd p) wX wY
+sendBundle :: forall rt p wX wY . (RepoPatch p, ApplyState p ~ Tree)
+           => [DarcsFlag] -> FL (PatchInfoAnd rt p) wX wY
              -> Doc -> String -> [WhatToDo] -> String -> IO ()
 sendBundle opts to_be_sent bundle fname wtds their_name=
          let
-           auto_subject :: forall pp wA wB . FL (PatchInfoAnd pp) wA wB -> String
+           auto_subject :: forall pp wA wB . FL (PatchInfoAnd rt pp) wA wB -> String
            auto_subject (p:>:NilFL)  = "darcs patch: " ++ trim (patchDesc p) 57
            auto_subject (p:>:ps) = "darcs patch: " ++ trim (patchDesc p) 43 ++
                             " (and " ++ show (lengthFL ps) ++ " more)"
@@ -453,8 +454,8 @@ cleanup opts (Just mailfile) = when (isNothing (hasLogfile opts) || willRemoveLo
                                       removeFileMayNotExist mailfile
 cleanup _ Nothing = return ()
 
-writeBundleToFile :: forall p wX wY . (RepoPatch p, ApplyState p ~ Tree)
-                  => [DarcsFlag] -> FL (PatchInfoAnd p) wX wY -> Doc ->
+writeBundleToFile :: forall rt p wX wY . (RepoPatch p, ApplyState p ~ Tree)
+                  => [DarcsFlag] -> FL (PatchInfoAnd rt p) wX wY -> Doc ->
                     AbsolutePathOrStd -> [WhatToDo] -> String -> IO ()
 writeBundleToFile opts to_be_sent bundle fname wtds their_name =
     do (d,f,_) <- getDescription opts their_name to_be_sent
@@ -470,7 +471,7 @@ data WhatToDo
     = Post String        -- ^ POST the patch via HTTP
     | SendMail String    -- ^ send patch via email
 
-decideOnBehavior :: RepoPatch p => [DarcsFlag] -> Maybe (Repository p wR wU wT) -> IO [WhatToDo]
+decideOnBehavior :: RepoPatch p => [DarcsFlag] -> Maybe (Repository rt p wR wU wT) -> IO [WhatToDo]
 decideOnBehavior opts remote_repo =
     case the_targets of
     [] -> do wtds <- case remote_repo of
@@ -522,7 +523,7 @@ collectTargets flags = [ f t | Target t <- flags ] where
     f em = SendMail em
 
 getDescription :: (RepoPatch p, ApplyState p ~ Tree)
-               => [DarcsFlag] -> String -> FL (PatchInfoAnd p) wX wY -> IO (Doc, Maybe String, Maybe String)
+               => [DarcsFlag] -> String -> FL (PatchInfoAnd rt p) wX wY -> IO (Doc, Maybe String, Maybe String)
 getDescription opts their_name patches =
     case get_filename of
         Just file -> do
