@@ -24,18 +24,20 @@ module Darcs.UI.Commands.WhatsNew
     , status
     ) where
 
-import Prelude hiding ( (^), catch )
+import Prelude ()
+import Darcs.Prelude
 
-import Control.Applicative ( (<$>) )
 import Control.Monad ( void )
 import Control.Monad.Reader ( runReaderT )
 import Control.Monad.State ( evalStateT, liftIO )
-import Storage.Hashed.Tree ( Tree )
+import Darcs.Util.Tree ( Tree )
 import System.Exit ( ExitCode (..), exitSuccess, exitWith )
+import Data.List.Ordered ( nubSort )
 
 import Darcs.Patch
     ( PrimOf, PrimPatch, RepoPatch
     , applyToTree, plainSummaryPrims, primIsHunk
+    , listTouchedFiles, IsRepoType
     )
 import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch.Choices ( patchChoicesLps, lpPatch )
@@ -60,16 +62,17 @@ import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 import Darcs.Patch.Witnesses.WZipper ( FZipper (..) )
 import Darcs.Repository
     ( RepoJob (..), Repository
-    , listRegisteredFiles, readRecorded
+    , listRegisteredFiles, readRecorded, readRepo
     , unrecordedChangesWithPatches, withRepository
     )
 import Darcs.Repository.Diff ( treeDiff )
 import Darcs.Repository.Prefs ( filetypeFunction )
-import Darcs.Repository.Util ( getMovesPs, getReplaces )
+import Darcs.Repository.State ( getMovesPs, getReplaces )
 import Darcs.UI.Commands
     ( DarcsCommand(..), withStdOpts, amInRepository
     , commandAlias, nodefaults
     )
+import Darcs.Repository.Resolution ( patchsetConflictResolutions )
 import Darcs.UI.Commands.Util ( announceFiles )
 import Darcs.UI.Flags
     ( DarcsFlag (Summary, LookForAdds, LookForMoves), diffAlgorithm, diffingOpts
@@ -94,7 +97,7 @@ import Darcs.UI.SelectChanges
     , skipOne, printSummary
     )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
-import Darcs.Util.Path ( AbsolutePath, SubPath, toFilePath )
+import Darcs.Util.Path ( AbsolutePath, SubPath, toFilePath, fp2fn )
 import Darcs.Util.Printer
     ( putDocLn, renderString, RenderMode(..)
     , text, vcat
@@ -105,6 +108,7 @@ import Darcs.Util.Prompt ( PromptConfig (..), promptChar )
 whatsnewBasicOpts :: DarcsOption a
                      (Maybe O.Summary
                       -> O.WithContext
+                      -> Bool
                       -> O.LookFor
                       -> O.DiffAlgorithm
                       -> Maybe String
@@ -113,17 +117,19 @@ whatsnewBasicOpts :: DarcsOption a
 whatsnewBasicOpts
     = O.summary
     ^ O.withContext
+    ^ O.machineReadable
     ^ O.lookfor
     ^ O.diffAlgorithm
     ^ O.workingRepoDir
     ^ O.interactive -- False
 
-whatsnewAdvancedOpts :: DarcsOption a (O.UseIndex -> Bool -> a)
+whatsnewAdvancedOpts :: DarcsOption a (O.UseIndex -> O.IncludeBoring -> a)
 whatsnewAdvancedOpts = O.useIndex ^ O.includeBoring
 
 whatsnewOpts :: DarcsOption a
                 (Maybe O.Summary
                  -> O.WithContext
+                 -> Bool
                  -> O.LookFor
                  -> O.DiffAlgorithm
                  -> Maybe String
@@ -134,7 +140,7 @@ whatsnewOpts :: DarcsOption a
                  -> O.Verbosity
                  -> Bool
                  -> O.UseIndex
-                 -> Bool
+                 -> O.IncludeBoring
                  -> O.UseCache
                  -> Maybe String
                  -> Bool
@@ -147,7 +153,6 @@ patchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = []
-    , S.diffAlgorithm = diffAlgorithm flags
     , S.interactive = isInteractive True flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
     , S.summary = hasSummary (defaultSummary flags) flags
@@ -155,7 +160,10 @@ patchSelOpts flags = S.PatchSelectionOptions
     }
 
 defaultSummary :: [DarcsFlag] -> O.Summary
-defaultSummary flags = if lookForAdds flags == O.YesLookForAdds then O.YesSummary else O.NoSummary
+defaultSummary flags
+  | lookForAdds flags == O.YesLookForAdds = O.YesSummary
+  | parseFlags O.machineReadable flags    = O.YesSummary
+  | otherwise = O.NoSummary
 
 whatsnew :: DarcsCommand [DarcsFlag]
 whatsnew = DarcsCommand
@@ -202,6 +210,10 @@ whatsnewHelp =
  "  conflict with a hunk in another patch.  The phrase `duplicated`\n" ++
  "  means the hunk is known to be identical to a hunk in another patch.\n" ++
  "\n" ++
+ "The `--machine-readable` option implies `--summary` while making it more\n" ++
+ "parsable. Modified files are only shown as `M f`, and moves are shown in\n" ++
+ "two lines: `F f` and `T g` (as in 'From f To g').\n" ++
+ "\n" ++
  "By default, `darcs whatsnew` uses Darcs' internal format for changes.\n" ++
  "To see some context (unchanged lines) around each change, use the\n" ++
  "`--unified` option.  To view changes in conventional `diff` format, use\n" ++
@@ -212,19 +224,19 @@ whatsnewHelp =
 
 whatsnewCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 whatsnewCmd fps opts args =
-   withRepository (useCache opts) $ RepoJob $ \(repo :: Repository p wR wU wR) -> do
+   withRepository (useCache opts) $ RepoJob $ \(repo :: Repository rt p wR wU wR) -> do
     files <- if null args
                  then return Nothing
                  else Just <$> fixSubPaths fps args
     let isLookForMoves = lookForMoves opts == O.YesLookForMoves && parseFlags O.summary opts /= Just O.NoSummary
         isLookForAdds = lookForAdds opts == O.YesLookForAdds && parseFlags O.summary opts /= Just O.NoSummary
         isLookForReplaces = lookForReplaces opts == O.YesLookForReplaces
+        isMachineReadable = parseFlags O.machineReadable opts
         -- LookForAdds and LookForMoves implies Summary, unless it's explcitly disabled.
-        optsModifier = if isLookForAdds
-                           then (Summary :) . filter (\o -> LookForAdds /= o &&
-                                                            LookForMoves /= o )
-                           else id
-        opts' = optsModifier opts
+        opts' | isLookForAdds     = (Summary : filter (\o -> LookForAdds /= o &&
+                                                             LookForMoves /= o ) opts)
+              | isMachineReadable = (Summary:opts)
+              | otherwise         = opts
     movesPs <- if isLookForMoves
         then getMovesPs repo files
         else return NilFL
@@ -246,12 +258,12 @@ whatsnewCmd fps opts args =
             -- Return the patches that create files/dirs that aren't yet added.
             unFreeLeft <$> treeDiff (diffAlgorithm opts) ftf noLookAddsTree lookAddsTree
         else return (Sealed NilFL)
-    announceFiles files "What's new in"
+    announceFiles (verbosity opts) files "What's new in"
     exitOnNoChanges (unaddedNewPathsPs, noLookChanges)
     if maybeIsInteractive opts
       then runInteractive (interactiveHunks pristine) opts' pristine noLookChanges
       else do
-        printChanges opts' pristine noLookChanges
+        printChanges repo opts' pristine noLookChanges
         printUnaddedPaths unaddedNewPathsPs
   where
     -- |Filter out hunk patches (leaving add patches) and return the tree
@@ -268,7 +280,7 @@ whatsnewCmd fps opts args =
     printUnaddedPaths :: PrimPatch p => FL p wX wY -> IO ()
     printUnaddedPaths NilFL = return ()
     printUnaddedPaths ps =
-        putDocLn . lowercaseAs . renderString Encode . plainSummaryPrims $ ps
+        putDocLn . lowercaseAs . renderString Encode . (plainSummaryPrims False []) $ ps
 
     -- Make any add markers lowercase, to distinguish new-but-unadded files
     -- from those that are unrecorded, but added.
@@ -277,24 +289,30 @@ whatsnewCmd fps opts args =
     lowercaseA x = x
 
     -- |Appropriately print changes, according to the passed flags.
-    printChanges :: (PatchListFormat p, IsHunk p, Patchy p, ShowPatch p, PrimDetails p,
-                 ApplyState p ~ Tree) => [DarcsFlag] -> Tree IO -> FL p wX wY
+    printChanges :: forall rt p wR wU wX wY. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
+                 => Repository rt p wR wU wR -> [DarcsFlag] -> Tree IO -> FL (PrimOf p) wX wY
                  -> IO ()
-    printChanges opts' pristine changes
-        | Summary `elem` opts' = putDocLn $ plainSummaryPrims changes
+    printChanges repo opts' pristine changes
+        | Summary `elem` opts' = do
+             r <- readRepo repo
+             Sealed res <- return $ patchsetConflictResolutions r
+             let conflictFns = map fp2fn $ nubSort $ listTouchedFiles res
+             putDocLn $ plainSummaryPrims machineReadable conflictFns changes
         | isUnified opts' == O.YesContext = contextualPrintPatch pristine changes
         | otherwise = printPatch changes
+     where machineReadable = parseFlags O.machineReadable opts
 
     -- |return the unrecorded changes that affect an optional list of paths.
-    filteredUnrecordedChanges :: forall p wR wU wT. (RepoPatch p, ApplyState p ~ Tree,
-                              ApplyState (PrimOf p) ~ Tree) => [DarcsFlag]
-                              -> Repository p wR wU wT -> Maybe [SubPath]
+    filteredUnrecordedChanges :: forall rt p wR wU wT. (RepoPatch p, ApplyState p ~ Tree,
+                                                        ApplyState (PrimOf p) ~ Tree)
+                              => [DarcsFlag]
+                              -> Repository rt p wR wU wT -> Maybe [SubPath]
                               -> FL (PrimOf p) wR wT -- look-for-moves patches
                               -> FL (PrimOf p) wT wT -- look-for-replaces patches
                               -> IO (Sealed (FL (PrimOf p) wT))
     filteredUnrecordedChanges  opts' repo files movesPs replacesPs =
         let filePaths = map toFilePath <$> files in
-        choosePreTouching filePaths <$> unrecordedChangesWithPatches (diffingOpts opts') repo files movesPs replacesPs
+        choosePreTouching filePaths <$> unrecordedChangesWithPatches movesPs replacesPs (diffingOpts opts') repo files
 
 -- | Runs the 'InteractiveSelectionM' code
 runInteractive :: (PatchListFormat p, IsHunk p, Patchy p, ShowPatch p,
@@ -314,7 +332,8 @@ runInteractive i opts pristine ps' = do
                  , choices = choices'
                  }
     void $ runReaderT ps $
-           selectionContextPrim First "view" (patchSelOpts opts) (Just primSplitter)
+           selectionContextPrim First "view" (patchSelOpts opts)
+             (Just (primSplitter (diffAlgorithm opts)))
              Nothing (Just pristine)
 
 -- | The interactive part of @darcs whatsnew@
