@@ -1,4 +1,3 @@
-{-# OPTIONS_GHC -fno-warn-dodgy-imports #-} -- needed for GHC 7.0/7.2
 {-# LANGUAGE BangPatterns, ForeignFunctionInterface, CPP, ScopedTypeVariables #-}
 
 -----------------------------------------------------------------------------
@@ -30,6 +29,8 @@ module Darcs.Util.ByteString (
         gzWriteFilePSs,
         gzReadStdin,
         gzWriteHandle,
+        FileSegment,
+        readSegment,
 
         -- gzip handling
         isGZFile,
@@ -60,10 +61,13 @@ module Darcs.Util.ByteString (
         decodeString
     ) where
 
-import Prelude hiding ( catch )
+import Prelude ()
+import Darcs.Prelude
+
 import qualified Data.ByteString            as B
 import qualified Data.ByteString.Char8      as BC
 import qualified Data.ByteString.Internal   as BI
+import qualified Data.ByteString.Lazy.Char8 as BL8
 import Data.ByteString (intercalate)
 import Data.ByteString.Internal (fromForeignPtr)
 
@@ -71,7 +75,10 @@ import Data.ByteString.Internal (fromForeignPtr)
 #else
 import Control.Exception ( catch, SomeException )
 #endif
-import System.IO
+import System.IO ( withFile, IOMode(ReadMode)
+                 , hSeek, SeekMode(SeekFromEnd,AbsoluteSeek)
+                 , openBinaryFile, hClose, Handle, hGetChar
+                 , stdin)
 import System.IO.Unsafe         ( unsafePerformIO )
 
 import Foreign.Storable         ( peek )
@@ -81,7 +88,7 @@ import Foreign.C.Types          ( CInt(..) )
 import Data.Bits                ( rotateL )
 import Data.Char                ( ord, isSpace )
 import Data.Word                ( Word8 )
-import Data.Int                 ( Int32 )
+import Data.Int                 ( Int32, Int64 )
 import qualified Data.Text as T ( pack, unpack )
 import Data.Text.Encoding       ( encodeUtf8, decodeUtf8With )
 import Data.Text.Encoding.Error ( lenientDecode )
@@ -110,6 +117,7 @@ import System.IO.MMap( mmapFileByteString )
 import System.Mem( performGC )
 import System.Posix.Files( fileSize, getSymbolicLinkStatus )
 #endif
+import qualified Bundled.Posix as Bundled ( getFileStatus, fileSize )
 
 -- -----------------------------------------------------------------------------
 -- obsolete debugging code
@@ -396,6 +404,29 @@ gzReadStdin = do
            in
            B.concat $ decompress compressed
 
+-- | Pointer to a filesystem, possibly with start/end offsets. Supposed to be
+-- fed to (uncurry mmapFileByteString) or similar.
+type FileSegment = (FilePath, Maybe (Int64, Int))
+
+-- | Read in a FileSegment into a Lazy ByteString. Implemented using mmap.
+readSegment :: FileSegment -> IO BL.ByteString
+readSegment (f,range) = do
+    bs <- tryToRead
+       `catch` (\(_::SomeException) -> do
+                     size <- Bundled.fileSize `fmap` Bundled.getFileStatus f
+                     if size == 0
+                        then return BC.empty
+                        else performGC >> tryToRead)
+    return $ BL8.fromChunks [bs]
+  where
+    tryToRead =
+        case range of
+            Nothing -> B.readFile f
+            Just (off, size) -> withFile f ReadMode $ \h -> do
+                hSeek h AbsoluteSeek $ fromIntegral off
+                B.hGet h size
+{-# INLINE readSegment #-}
+
 -- -----------------------------------------------------------------------------
 -- mmapFilePS
 
