@@ -23,8 +23,7 @@
 
 {-# LANGUAGE CPP #-}
 module Darcs.Util.Path
-    ( module Storage.Hashed.AnchoredPath
-    , FileName( )
+    ( FileName( )
     , fp2fn
     , fn2fp
     , fn2ps
@@ -74,16 +73,28 @@ module Darcs.Util.Path
     -- * Tree filtering.
     , filterFilePaths
     , filterPaths
-    ) where
+    -- * AnchoredPaths: relative paths within a Tree. All paths are
+    -- anchored at a certain root (this is usually the Tree root). They are
+    -- represented by a list of Names (these are just strict bytestrings).
+    , Name(..)
+    , AnchoredPath(..)
+    , anchoredRoot
+    , appendPath
+    , anchorPath
+    , isPrefix
+    , parent, parents, catPaths, flatten, makeName, appendToName
+    -- * Unsafe AnchoredPath functions.
+    , floatBS, floatPath, replacePrefixPath ) where
 
-import Storage.Hashed.AnchoredPath
+import Prelude ()
+import Darcs.Prelude
 
-import Control.Applicative ( (<$>) )
 import Data.List
     ( isPrefixOf
     , isSuffixOf
     , stripPrefix
     , intersect
+    , inits
     )
 import Data.Char ( isSpace, chr, ord )
 import Control.Exception ( tryJust, bracket_ )
@@ -95,19 +106,20 @@ import qualified System.Directory ( setCurrentDirectory )
 import System.Directory ( doesDirectoryExist, doesFileExist )
 import qualified System.FilePath.Posix as FilePath ( normalise )
 import qualified System.FilePath as NativeFilePath ( takeFileName, takeDirectory )
-import System.FilePath ( splitDirectories )
+import System.FilePath( (</>), splitDirectories, normalise, dropTrailingPathSeparator )
 import System.Posix.Files ( isDirectory, getSymbolicLinkStatus )
 
 import Darcs.Util.ByteString ( packStringToUTF8, unpackPSFromUTF8 )
-import qualified Data.ByteString.Char8 as BC (unpack, pack)
+import qualified Data.ByteString.Char8 as BC
 import qualified Data.ByteString       as B  (ByteString)
-import Data.Binary
 
+import Data.Binary
 import Darcs.Util.Global ( darcsdir )
 import Darcs.Util.URL ( isAbsolute, isRelative, isSshNopath )
 
 #include "impossible.h"
 
+
 -- | FileName is an abstract type intended to facilitate the input and output of
 -- unicode filenames.
 newtype FileName = FN FilePath deriving ( Eq, Ord )
@@ -261,10 +273,10 @@ movedirfilename old new name =
 
 
 class FilePathOrURL a where
- toPath :: a -> String
+  toPath :: a -> String
 
 class FilePathOrURL a => FilePathLike a where
- toFilePath :: a -> FilePath
+  toFilePath :: a -> FilePath
 
 -- | Paths which are relative to the local darcs repository and normalized.
 -- Note: These are understood not to have the dot in front.
@@ -279,35 +291,35 @@ data AbsolutePathOrStd = AP AbsolutePath | APStd deriving (Eq, Ord)
 data AbsoluteOrRemotePath = AbsP AbsolutePath | RmtP String deriving (Eq, Ord)
 
 instance FilePathOrURL AbsolutePath where
- toPath (AbsolutePath x) = x
+  toPath (AbsolutePath x) = x
 instance FilePathOrURL SubPath where
- toPath (SubPath x) = x
+  toPath (SubPath x) = x
 instance CharLike c => FilePathOrURL [c] where
- toPath = toFilePath
+  toPath = toFilePath
 
 instance FilePathOrURL AbsoluteOrRemotePath where
- toPath (AbsP a) = toPath a
- toPath (RmtP r) = r
+  toPath (AbsP a) = toPath a
+  toPath (RmtP r) = r
 
 instance FilePathOrURL FileName where
-    toPath = fn2fp
+  toPath = fn2fp
 instance FilePathLike FileName where
-    toFilePath = fn2fp
+  toFilePath = fn2fp
 
 instance FilePathLike AbsolutePath where
- toFilePath (AbsolutePath x) = x
+  toFilePath (AbsolutePath x) = x
 instance FilePathLike SubPath where
- toFilePath (SubPath x) = x
+  toFilePath (SubPath x) = x
 
 class CharLike c where
-    toChar :: c -> Char
-    fromChar :: Char -> c
+  toChar :: c -> Char
+  fromChar :: Char -> c
 instance CharLike Char where
-    toChar = id
-    fromChar = id
+  toChar = id
+  fromChar = id
 
 instance CharLike c => FilePathLike [c] where
-    toFilePath = map toChar
+  toFilePath = map toChar
 
 -- | Make the second path relative to the first, if possible
 makeSubPathOf :: AbsolutePath -> AbsolutePath -> Maybe SubPath
@@ -552,4 +564,101 @@ getUniquePathName talkative buildMsg buildName = go (-1)
 -- | Transform a SubPath into an AnchoredPath.
 floatSubPath :: SubPath -> AnchoredPath
 floatSubPath = floatPath . fn2fp . sp2fn
+ 
+-------------------------------
+-- AnchoredPath utilities
+--
 
+newtype Name = Name BC.ByteString  deriving (Eq, Show, Ord)
+
+-- | This is a type of "sane" file paths. These are always canonic in the sense
+-- that there are no stray slashes, no ".." components and similar. They are
+-- usually used to refer to a location within a Tree, but a relative filesystem
+-- path works just as well. These are either constructed from individual name
+-- components (using "appendPath", "catPaths" and "makeName"), or converted
+-- from a FilePath ("floatPath" -- but take care when doing that) or .
+newtype AnchoredPath = AnchoredPath [Name] deriving (Eq, Show, Ord)
+
+-- | Check whether a path is a prefix of another path.
+isPrefix :: AnchoredPath -> AnchoredPath -> Bool
+(AnchoredPath a) `isPrefix` (AnchoredPath b) = a `isPrefixOf` b
+
+-- | Append an element to the end of a path.
+appendPath :: AnchoredPath -> Name -> AnchoredPath
+appendPath (AnchoredPath p) n =
+    case n of
+      (Name s) | s == BC.empty -> AnchoredPath p
+               | s == BC.pack "." -> AnchoredPath p
+               | otherwise -> AnchoredPath $ p ++ [n]
+
+-- | Catenate two paths together. Not very safe, but sometimes useful
+-- (e.g. when you are representing paths relative to a different point than a
+-- Tree root).
+catPaths :: AnchoredPath -> AnchoredPath -> AnchoredPath
+catPaths (AnchoredPath p) (AnchoredPath n) = AnchoredPath $ p ++ n
+
+-- | Get parent (path) of a given path. foo/bar/baz -> foo/bar
+parent :: AnchoredPath -> AnchoredPath
+parent (AnchoredPath x) = AnchoredPath (init x)
+
+-- | List all parents of a given path. foo/bar/baz -> [foo, foo/bar]
+parents :: AnchoredPath -> [AnchoredPath]
+parents (AnchoredPath x) = map AnchoredPath . init . inits $ x
+
+-- | Take a "root" directory and an anchored path and produce a full
+-- 'FilePath'. Moreover, you can use @anchorPath \"\"@ to get a relative
+-- 'FilePath'.
+anchorPath :: FilePath -> AnchoredPath -> FilePath
+anchorPath dir p = dir </> BC.unpack (flatten p)
+{-# INLINE anchorPath #-}
+
+-- | Unsafe. Only ever use on bytestrings that came from flatten on a
+-- pre-existing AnchoredPath.
+floatBS :: BC.ByteString -> AnchoredPath
+floatBS = AnchoredPath . map Name . takeWhile (not . BC.null) . BC.split '/'
+
+flatten :: AnchoredPath -> BC.ByteString
+flatten (AnchoredPath []) = BC.singleton '.'
+flatten (AnchoredPath p) = BC.intercalate (BC.singleton '/')
+                                           [ n | (Name n) <- p ]
+
+makeName :: String -> Name
+makeName ".." = error ".. is not a valid AnchoredPath component name"
+makeName n | '/' `elem` n = error "/ may not occur in a valid AnchoredPath component name"
+           | otherwise = Name $ BC.pack n
+
+-- | Take a relative FilePath and turn it into an AnchoredPath. The operation
+-- is (relatively) unsafe. Basically, by using floatPath, you are testifying
+-- that the argument is a path relative to some common root -- i.e. the root of
+-- the associated "Tree" object. Also, there are certain invariants about
+-- AnchoredPath that this function tries hard to preserve, but probably cannot
+-- guarantee (i.e. this is a best-effort thing). You should sanitize any
+-- FilePaths before you declare them "good" by converting into AnchoredPath
+-- (using this function).
+floatPath :: FilePath -> AnchoredPath
+floatPath = make . splitDirectories . normalise . dropTrailingPathSeparator
+  where make ["."] = AnchoredPath []
+        make x = AnchoredPath $ map (Name . BC.pack) x
+
+
+anchoredRoot :: AnchoredPath
+anchoredRoot = AnchoredPath []
+
+-- | Take a prefix path, the changed prefix path, and a path to change.
+-- Assumes the prefix path is a valid prefix. If prefix is wrong return
+-- AnchoredPath [].
+replacePrefixPath :: AnchoredPath -> AnchoredPath -> AnchoredPath -> AnchoredPath
+replacePrefixPath (AnchoredPath []) b c = catPaths b c
+replacePrefixPath (AnchoredPath (r:p)) b (AnchoredPath (r':p'))
+    | r == r' = replacePrefixPath (AnchoredPath p) b (AnchoredPath p')
+    | otherwise = AnchoredPath []
+replacePrefixPath _ _ _ = AnchoredPath []
+
+-- | Append a ByteString to the last Name of an AnchoredPath.
+appendToName :: AnchoredPath -> String -> AnchoredPath
+appendToName (AnchoredPath p) s = AnchoredPath (init p++[Name finalname])
+    where suffix = BC.pack s
+          finalname | suffix `elem` (BC.tails lastname) = lastname
+                    | otherwise = BC.append lastname suffix
+          lastname = case last p of
+                        Name name -> name
